#include <GSM.h>

#define PINNUMMER "0155"
#define TELEFONNUMMER "01747778983"
#define WARTEZEIT 60*60*1000
#define IRSENSORPIN A0
#define DEBUG true

GSM gsmAccess;
GSM_SMS sms;

void InitGSM()
{
  Serial.println("GSM wird initialisiert");
  if (!DEBUG) {
    boolean KeineVerbindung = true;

    while (KeineVerbindung)
    {
      if (gsmAccess.begin(PINNUMMER) == GSM_READY)
        KeineVerbindung = false;
      else
      {
        Serial.println("GSM-Fehler: keine Verbindung");
        delay(1000);
      }
    }
  }
  Serial.println("GSM wurde erfolgreich gestartet");
}

void SendSMS(char* SMSText, char* Telefonnummer)
{
  Serial.print("SMS senden an ");
  Serial.println(Telefonnummer);
  if (!DEBUG) {
    sms.beginSMS(Telefonnummer);
    sms.print(SMSText);
    sms.endSMS();
  }
  Serial.println("SMS gesendet");
}

bool PruefeBewegungssensor()
{
  return digitalRead(IRSENSORPIN);
}

void setup()
{
  pinMode(IRSENSORPIN, INPUT);
  Serial.begin(9600);
  Serial.println("Arduino Alarmanlage");
  InitGSM();
}

long LetzterAlarm = -WARTEZEIT; //

void loop()
{
  if ( PruefeBewegungssensor() ) {
    if ((millis() - LetzterAlarm) >= WARTEZEIT)
    {
      Serial.println("Alarm! Sende SMS");
      LetzterAlarm = millis();
      SendSMS("Alarm! Es wird bei Ihnen gerade eingebrochen!", TELEFONNUMMER);
    }
    else {
      Serial.println("Alarm! Aber SMS wurde bereits gesendet.");
    }
  }
  else {
    Serial.println("Alles Ruhig");
  }
  delay(500);
}

