/*     A
 *    ---
 *F |   | B
 *   --- G
 *E |   | C
 *   --- . DP
 *    D
 *
 */

//                  A B C D E F G DP
int Nummern[11][8] = {
  {
    1, 1, 1, 1, 1, 1, 0, 0
  }
  , // 0
  {
    0, 1, 1, 0, 0, 0, 0, 0
  }
  , // 1
  {
    1, 1, 0, 1, 1, 0, 1, 0
  }
  , // 2
  {
    1, 1, 1, 1, 0, 0, 1, 0
  }
  , // 3
  {
    0, 1, 1, 0, 0, 1, 1, 0
  }
  , // 4
  {
    1, 0, 1, 1, 0, 1, 1, 0
  }
  , // 5
  {
    1, 0, 1, 1, 1, 1, 1, 0
  }
  , // 6
  {
    1, 1, 1, 0, 0, 0, 0, 0
  }
  , // 7
  {
    1, 1, 1, 1, 1, 1, 1, 0
  }
  , // 8
  {
    1, 1, 1, 1, 0, 1, 1, 0
  }
  , // 9
  {
    0, 0, 0, 0, 0, 0, 0, 0
  }
};//nichts

const int button_reset = 11;   //Zum Einstellen der Countdown-Zeit
const int button_start = 12; //Zum Starten des Countdowns / der Stopuhr

//Initialisiere das Programm
void setup() {
  //Setze die Pins für die Ansteuerung der Segmente als OUTPUT
  for (int i = 2; i < 2 + 8; i++)
  {
    pinMode(i, OUTPUT);
    digitalWrite(i, LOW);
  }

  //Setze die Pins für die Ansteuerung der gemeinsamen Anode/Kathode als OUTPUT
  pinMode(A1, OUTPUT);
  pinMode(A2, OUTPUT);
  pinMode(A3, OUTPUT);
  pinMode(A4, OUTPUT);


  //Setze die Pins für die Taster als INPUT
  pinMode(button_reset, INPUT);
  pinMode(button_start, INPUT);

  //Schalte die PullUp-Widerstände für die Taster an
  digitalWrite(button_reset, HIGH);
  digitalWrite(button_start, HIGH);
}

//Forme mit den Segment-Anzeigen eine Nummer
//
// Ziffer - eine Zahl von 0 bis 10, 10 ist hierbei ein leeres Feld
// Slot_Pin - die gemeinsame Anode oder Kathode der Ziffer
// setzte_Punkt - soll zusätzlich zur dargestellten Ziffern auch der Punkt eingeschaltet werden?
void schreibe_Ziffer(byte Ziffer, byte Slot_Pin, boolean setzte_Punkt)
{

  //Schalte die Segmente an, die für das Darstellen der Ziffer benötigt werden
  for (int i = 0; i < 7; i++)
  {
    digitalWrite(i + 2, !Nummern[Ziffer][i]);
  }

  //Schalte wenn gefordert auch das Segment für den Punkt an
  digitalWrite(7 + 2, (setzte_Punkt ? LOW : HIGH));

  //Schalte die gemeinsame Anode/Kathode ein, alle vorher eingeschalteten Segmente leuchten.
  digitalWrite(Slot_Pin, HIGH);

  //Warte eine Millisekunde ab
  delay(1);

  //Schalte die gemeinsame Anode/Kathode wieder aus.
  digitalWrite(Slot_Pin, LOW);
}

//Schreibe eine vierstellige Zahl auf die Sieben-Segment-Anzeige.
//Die Methode schreibt die Nummer nur ein Mal, für ein dauerhaftes darstellen muss sie
//wiederholt aufgerufen werden.
//
// Zahl - die darzustellende Nummer
// leading_Zero - sollen führende Nullen dargestellt werden? ("0123" oder " 123")
// setze_Punkt - sollen die zwei Punkte in der Mitte eingeschaltet werden?
void schreibe_Zahl(int Zahl, boolean fuehrende_Null, boolean setze_Punkt)
{
  //Schreibe die niedrigste Stelle der Zahl ("1234" -> "4")
  if (Zahl >= 0 || !fuehrende_Null)
  {
    schreibe_Ziffer(Zahl % 10, A4, false);
  }
  else
  {
    schreibe_Ziffer(10, A4, false);
  }

  //Schreibe die zweit-niedrigste Stelle der Zahl ("1234" -> "3")
  if (Zahl >= 10 || !fuehrende_Null)
  {
    schreibe_Ziffer((Zahl / 10) % 10, A3, setze_Punkt);
  }
  else
  {
    schreibe_Ziffer(10, A3, setze_Punkt);
  }

  //Schreibe die dritt-niedrigste Stelle der Zahl ("1234" -> "2")
  if (Zahl >= 100 || !fuehrende_Null)
  {
    schreibe_Ziffer((Zahl / 100) % 10, A2, setze_Punkt);
  }
  else
  {
    schreibe_Ziffer(10, A2, setze_Punkt);
  }

  //Schreibe die viert-niedrigste Stelle der Zahl ("1234" -> "1")
  if (Zahl >= 1000 || !fuehrende_Null)
  {
    schreibe_Ziffer((Zahl / 1000) % 10, A1, false);
  }
  else
  {
    schreibe_Ziffer(10, A1, false);
  }
}

boolean running = false; //Wurde die Stopuhr / der Countdown schon gestartet?
int Sekunden = 0; //Darzustellende Minuten
int Minuten = 0; //darzustellende Sekunden
int derzeitige_Wartezeit = 0; //Wielange bis zum nächsten gültigen Tastendruck?
int Wartezeit = 100; //Wie viele Zyklen müssen zwischen zwei Tastendrücken vergehen?
long Startzeit;

void loop() {

  //Darf gerade ein Taster gedrückt werden?
  if (derzeitige_Wartezeit == 0)
  {
    //Tastendruck START/STOP
    if (!digitalRead(button_start))
    {
      running = !running;
      Startzeit = millis() / 1000 - Sekunden - Minuten * 60;

      derzeitige_Wartezeit = Wartezeit;
    }

    //Tastendruck RESET
    if (!digitalRead(button_reset)) {
      running = false;
      Sekunden = 0;
      Minuten = 0;
    }
  }
  else
  {
    derzeitige_Wartezeit--;
  }

  if (running) {
    //Wie viel Zeit (in Sekunden) ist seit dem Starten der Stoppuhr vergangen?
    long time = millis() / 1000 - Startzeit;
    Sekunden = time % 60;
    Minuten = time / 60;
  }

  //Schreibe die Zeit auf die Sieben-Segment-Anzeige
  schreibe_Zahl(Minuten * 100 + Sekunden, false, (millis() % 1000) < 500 || !running);
}

