#include <LiquidCrystal.h>

byte Omega[8] = {
  0b00000,  //
  0b00000,  //
  0b01110,  //  XXXXXX
  0b10001,  //XX      XX
  0b10001,  //XX      XX
  0b01010,  //  XX  XX
  0b11011,  //XXXX  XXXX
  0b00000   //
};

LiquidCrystal Lcd(12, 11, 5, 4, 3, 2);
const int PinMessung = A0;    //Pin, der für Messung verwendet wird
const int Messdauer = 1000;   //Wie viele Messungen pro Zeitschritt?
float Messwiderstand = 270;   //Ohm-Wert des bekannten Widerstands

float WiderstandAusmessen() {
  float Summe = 0.0f;                //Summiere über alle Messungen
  for (int i = 0; i < Messdauer; i++) {
    float Messwert = analogRead(PinMessung);
    //Wandle den 10bit-Wert (0-1023) in eine Spannung um.
    float GemesseneSpannung = Messwert / 1023.0f * 5.0f;
    //Berechne R2 = (Uges * R1) / (U1 - R1)
    float Resultat = Messwiderstand * (5.0f / GemesseneSpannung - 1.0f);
    Summe += Resultat;
  }
  return (Summe / (float)Messdauer); //Berechne den Durchschnitt 
}

void setup() {
  pinMode(PinMessung, INPUT);
  Lcd.begin(16, 2);  //Wieviele Spalten und Zeilen hat das Display?
  Lcd.createChar(0, Omega);    //Erstelle das Ohm-Zeichen
}

void loop() {
  //Warte, um Flimmern des Displays zu verhindern.
  delay(100);

  //Berechne den Wert des Widerstands R2
  int Messergebnis = (int)WiderstandAusmessen();

  Lcd.clear();               //Löschen den Inhalt des Displays.
  Lcd.setCursor(0, 0);       //Schreibe oben links.
  Lcd.print("Widerstand:");
  Lcd.setCursor(0, 1);       //Schreibe unten links.
  
  if (Messergebnis > 0)
  {
    Lcd.print(Messergebnis); //Schreibe das Ergebnis.
    Lcd.write((byte)0);      //Zeichne das Ohm-Zeichen
  }
}


