struct ball
{
  int x, y;
  int geschwindigkeit_x, geschwindigkeit_y;
  int geschwindigkeit;
};

struct schlaeger
{
  int x, y;
  int breite, hoehe;
};

struct block
{
  int x, y;
  int punktwert;
  boolean im_spiel;
  byte red, green, blue;
};

#include <UTFT.h>

//Die Schriftart, mit der Text angezeigt wird.
extern uint8_t BigFont[];

//Hier wird ein neues Display-Objekt initialisiert.
const int spi_CS = A10;
const int spi_SCK = 76;
const int spi_MOSI = 75;
const int spi_DC = A8;
const int spi_RST = A9;
UTFT myGLCD(ILI9341_S5P, spi_MOSI, spi_SCK,
            spi_CS, spi_RST, spi_DC);

const int pin_links = 24; //Der linke Knopf.
const int pin_rechts = 22;//Der rechte Knopf.

struct ball ball;
struct schlaeger schlaeger;
struct block bloecke[10 * 5];
int schlaeger_geschwindigkeit = 7;

void setup()
{
  //Setze den Ball auf das Spielfeld.
  ball.x = 160;
  ball.y = 100;
  ball.geschwindigkeit_x = 2;
  ball.geschwindigkeit_y = 6;
  ball.geschwindigkeit = ball.geschwindigkeit_x + ball.geschwindigkeit_y;

  //Setze den Schläger in die Mitte des Spielfelds
  schlaeger.x = 320 / 2;
  schlaeger.y = 240 - 20;
  schlaeger.breite = 70;
  schlaeger.hoehe = 2;
  
    //Erstelle die Blöcke auf dem Spielfeld.
  for (int i = 0; i < 10; i++) {
    for (int j = 0; j < 5; j++) {
      bloecke[i + j * 10].x = i * 32;
      bloecke[i + j * 10].y = j * 16;
      bloecke[i + j * 10].im_spiel = true;
      bloecke[i + j * 10].punktwert = 1;
      bloecke[i + j * 10].red = random(64, 150);
      bloecke[i + j * 10].green = random(64, 150);
      bloecke[i + j * 10].blue = random(64, 150);
    }
  }
  
    //Initialisiere die Knöpfe.
  pinMode(pin_links, INPUT);
  pinMode(pin_rechts, INPUT);
  digitalWrite(pin_links, HIGH);
  digitalWrite(pin_rechts, HIGH);

  //Initialisiere das Display
  myGLCD.InitLCD();
  myGLCD.setFont(BigFont);
  myGLCD.setBackColor(0, 0, 0);
  myGLCD.clrScr();
  
    //Zeichne alle Blöcke.
  myGLCD.setColor(155, 155, 155);
  myGLCD.setBackColor(255, 255, 255);
  for (int i = 0; i < 10 * 5; i++)
  {
    zeichne_block(bloecke[i]);
  }

  //Zeichne den Ball
  myGLCD.setColor(255, 255, 0);
  myGLCD.fillRect(ball.x - 2, ball.y - 2, ball.x + 2, ball.y + 2);

  //Zeichne den Schläger
  zeichne_schlaeger();

  delay(1000);
}

void zeichne_schlaeger() {
  //Überzeichne die alte Possition des Schlägers
  myGLCD.setColor(0, 0, 0);
  myGLCD.fillRect(schlaeger.x - schlaeger.breite / 2, schlaeger.y, schlaeger.x + schlaeger.breite / 2, schlaeger.y + schlaeger.hoehe);

  //Bewege den Schläger
  if (!digitalRead(pin_links))
    schlaeger.x = max(schlaeger.x - schlaeger_geschwindigkeit, schlaeger.breite / 2);
  if (!digitalRead(pin_rechts))
    schlaeger.x = min(schlaeger.x + schlaeger_geschwindigkeit, 320 - schlaeger.breite / 2);

  //Zeichne den Schläger an die neue Position.
  myGLCD.setColor(50, 150, 100);
  myGLCD.fillRect(schlaeger.x - schlaeger.breite / 2, schlaeger.y, schlaeger.x + schlaeger.breite / 2, schlaeger.y + schlaeger.hoehe);
}

//Zeichne einen Block im Spielfeld.
void zeichne_block(struct block block)
{
  if (block.im_spiel) {
    //Zeichne das Innere des Blocks.
    myGLCD.setColor(block.red, block.green, block.blue);
    myGLCD.fillRect(block.x, block.y, 31 + block.x, 15 + block.y);
    //Zeichne einen weißen Rand um das Innere.
    myGLCD.setColor(155, 155, 155);
    myGLCD.drawRect(block.x, block.y, 31 + block.x, 15 + block.y);
  } else {
    //Wenn der Block sich nicht im SPiel befindet, überzeichne ihn.
    myGLCD.setColor(0, 0, 0);
    myGLCD.fillRect(block.x, block.y, 31 + block.x, 15 + block.y);
  }
}

void bewege_ball() {

  //Berechne die neue Ballposition.
  ball.x += ball.geschwindigkeit_x;
  ball.y += ball.geschwindigkeit_y;

  //Wenn der Ball an edie linke, rechte oder obere Kante
  //stößt, dann drehe seine Richtung um.
  if (ball.x - 2 <= 0 || ball.x + 2 >= 320) {
    ball.geschwindigkeit_x = -ball.geschwindigkeit_x;
  }
  if (ball.y - 2 <= 0) {
    ball.geschwindigkeit_y = -ball.geschwindigkeit_y;
  }
}

//Prüfe, ob der Ball einen Block getroffen hat, und reagiere darauf.
boolean pruefe_blockkollision(struct block block)
{
  //Wie weit ist der Ball vom Block entfernt?
  int distanzX = max(block.x - ball.x, ball.x - block.x - 31);
  int distanzY = max(block.y - ball.y, ball.y - block.y - 15);

  if (distanzX < 2 && distanzY < 2)
  {
    //Lasse den Ball vom Block abprallen.
    if (distanzX > distanzY)
    {
      ball.geschwindigkeit_x = -ball.geschwindigkeit_x;
    }
    else
    {
      ball.geschwindigkeit_y = -ball.geschwindigkeit_y;
    }
    return true;
  }
  return false;
}

//Überprüfe, ob einer der Blöcke getroffen wurde
void pruefe_bloeckekollision() {
  boolean kollision_stattgefunden = false;

  //Überprüfe, ob der Ball einen Block getroffen hat.
  for (int i = 0; i < 10 * 5; i++)
  {
    if (bloecke[i].im_spiel)
      if (pruefe_blockkollision(bloecke[i]))
      {
        //Stelle sicher, dass immer nur ein Block
        //getroffen werden kann
        if (!kollision_stattgefunden) {
          //Entferne den Block aus dem Spiel.
          bloecke[i].im_spiel = false;
        }
        zeichne_block(bloecke[i]);
        kollision_stattgefunden = true;
      }
  }
}

void pruefe_schlaeger() {
  //Ist der Ball in der Höhe des Spielfelds, in dem der Schläger ist?
  if (ball.y > schlaeger.y - 2) {
    //Wo befindet sich der Ball, links oder rechts?
    int distanz = ball.x - schlaeger.x;
    //Wenn der Ball vom Schläger getroffen wird, dann:
    if (abs(distanz) < schlaeger.breite / 2 + 2) {
      //Lasse den Ball in die Breite zurückspringen, je weiter am
      //Rand des Schlägers er getroffen wurde desto mehr wird er
      //zur Seite hin beschleunigt.
      ball.geschwindigkeit_x = min((ball.geschwindigkeit - 2) *
                                   abs(distanz) / (schlaeger.breite / 2) + 1,
                                   ball.geschwindigkeit - 2);

      if (distanz < 0)
        ball.geschwindigkeit_x = -ball.geschwindigkeit_x;
      //Berechne, wie weit der Ball nach oben beschleunigt wird.
      ball.geschwindigkeit_y = - (ball.geschwindigkeit -
                                  abs(ball.geschwindigkeit_x));
    }
  }
}

void pruefe_gameOver() {
  boolean bloecke_im_feld = false;
  //Befindet sich noch ein Block auf dem Spielfeld?
  for (int i = 0; i < 10 * 5; i++) {
    if (bloecke[i].im_spiel)
      bloecke_im_feld = true;
  }
  if (ball.y > 240 || !bloecke_im_feld) {
    //Wenn der Ball aus dem Spielfeld verschwinden ist oder
    //sich keine Blöcke mehr im Spiel befinden,
    //dann zeige den Spielstand an.
    printGameScreen();
  }
}

//Zeichne den Bildschirm mit der erreichten Punktezahl.
void printGameScreen() {
  //Lösche das Spielfeld vom Bildschirm
  myGLCD.setBackColor(0, 0, 0);
  myGLCD.clrScr();
  //Schreibe "GAME OVER!"
  myGLCD.setColor(255, 255, 255);
  myGLCD.print("GAME OVER!", CENTER, 100);

  //Berechne die Punktzahl
  int score = 0;
  for (int i = 0; i < 10 * 5; i++) {
    if (!bloecke[i].im_spiel) {
      score += bloecke[i].punktwert;
    }
  }

  //Schreibe die Punktzahl auf den Bildschirm.
  myGLCD.print("Score: ", CENTER, 150);
  myGLCD.print(String(score), CENTER, 175);
  //Warte auf einen Reset.
  while (true) {}
}

void loop()
{
  //Überzeichne den Ball mit schwarz.
  myGLCD.setColor(0, 0, 0);
  myGLCD.fillRect(ball.x - 2, ball.y - 2, ball.x + 2, ball.y + 2);


  bewege_ball();
  pruefe_bloeckekollision();

  //Zeichne den Schläger und überprüfe, ob der Ball
  //ihn getroffen hat.
  zeichne_schlaeger();
  pruefe_schlaeger();

  pruefe_gameOver();

  //Zeichne den Ball an seine neue Position
  myGLCD.setColor(255, 255, 0);
  myGLCD.fillRect(ball.x - 2, ball.y - 2, ball.x + 2, ball.y + 2);

  //Warte ein bisschen, damit das Spiel zu nicht schnell läuft
  //und das Spielfeld lange genug gesehen werden kann.
  delay(20);
}

