const int Pin_Apwm = 3;  //Kanal A einschalten
const int Pin_Bpwm = 11; //Kanel B einschalten
const int Pin_Adir = 12; //Richtung Kanal A
const int Pin_Bdir = 13; //Richtung Kanal B
const int Pin_Abrk = 9;  //Bremse Kanal A
const int Pin_Bbrk = 8;  //Bremse Kanal B
const int Pin_Taster = 7;//Taster
const int Pin_Masse = 5; //Masse für den Taster
const int Pin_LED = A4;  //IR-LED Sensor
const int Pin_FotoTransistor = A3; //Sensor

//Array, in dem gespeichert ist, welche Pins
//bei welchem Schritt angeschaltet sein müssen
//Reihenfolge: Apwm, Abrk, Bpwm, Bbrk
const boolean Schrittfolge[8][4] =
{ {1, 1, 1, 1}, //+- +-
  {1, 1, 0, 0}, //+- 00
  {1, 1, 0, 1}, //+- -+
  {0, 0, 0, 1}, //00 -+
  {0, 1, 0, 1}, //-+ -+
  {0, 1, 0, 0}, //-+ 00
  {0, 1, 1, 1}, //-+ +-
  {0, 0, 1, 1}  //00 +-
};

//Ab wann soll die Uhr zurückgestellt werden?
const int langer_Tastendruck = 1000;
//Welcher Schritt wurde desletzt ausgeführt?
byte Zustand = 0; 
boolean Motor_ist_an = false;
unsigned int Taster_Startzeit = 0;
boolean Taster_gedrueckt = false;
boolean Uhr_laeuft = false;
//Wie viel Zeit vergeht zwischen jedem Schritt? Bei Motoren mit
//400 Halbschritten, die über 60 Sekunden verteilt sind:
unsigned long Zeit_pro_Tick = 150;//ms
unsigned long Zeit = 0;//ms

void setup() {
  //Initialisiere die Pins
  pinMode(Pin_Apwm, OUTPUT);
  pinMode(Pin_Bpwm, OUTPUT);
  pinMode(Pin_Adir, OUTPUT);
  pinMode(Pin_Bdir, OUTPUT);
  pinMode(Pin_Abrk, OUTPUT);
  pinMode(Pin_Bbrk, OUTPUT);
  pinMode(Pin_Taster, INPUT);
  pinMode(Pin_Masse, OUTPUT);
  pinMode(Pin_LED, OUTPUT);
  pinMode(Pin_FotoTransistor, INPUT);

  digitalWrite(Pin_Apwm, LOW);
  digitalWrite(Pin_Bpwm, LOW);
  digitalWrite(Pin_Adir, LOW);
  digitalWrite(Pin_Bdir, LOW);
  digitalWrite(Pin_Abrk, LOW);
  digitalWrite(Pin_Bbrk, LOW);
  digitalWrite(Pin_Taster, HIGH);
  digitalWrite(Pin_Masse, LOW);
  digitalWrite(Pin_LED, LOW);
  digitalWrite(Pin_FotoTransistor, LOW);

  //Starte mit ausgeschaltetem Motor
  schalte_Motor(false);
}

void loop() {
  if (!digitalRead(Pin_Taster)) { //Taster gedrückt?
    if (!Taster_gedrueckt) {
      Taster_gedrueckt = true;
      Taster_Startzeit = millis();
    }
  } else if (Taster_gedrueckt) { //Taster losgelassen?
    Taster_gedrueckt = false;
    //Wielange wurde der Schalter gedrückt?
    int Tastendauer = millis() - Taster_Startzeit;
    if (Tastendauer > langer_Tastendruck) {
      //Stoppe und setze die Uhr auf die 12-Uhr-Position
      Uhr_laeuft = false;
      zuruecksetzen();
      schalte_Motor(false);
    } else if(Tastendauer > 100) {
      //Stoppe oder lasse die Uhr laufen
      Uhr_laeuft = !Uhr_laeuft;
      schalte_Motor(Uhr_laeuft);
      Zeit = millis();
    }
  }

  if (Uhr_laeuft) {
    //Ist es Zeit, einen Schritt voranzuschreiten?
    if (millis() > (Zeit + Zeit_pro_Tick)) {
      schritt(true);
      Zeit += Zeit_pro_Tick;
    }
  }
}

void zuruecksetzen() {
  schalte_Motor(true);
  //Lasse die Uhr solange rotieren, bis 
  //der Sensor gefunden wurde
  while(Messe_Sensor() < 30) {
    schritt(false);
    delay(5);
    }
}

int Messe_Sensor() {
  //Messe den Sensorwert ohne LED
  int ohne_Licht = analogRead(Pin_FotoTransistor);
  digitalWrite(Pin_LED, HIGH);
  //Messe den Sensorwert mit LED
  int mit_Licht = analogRead(Pin_FotoTransistor);
  digitalWrite(Pin_LED, LOW);  
  //Wenn sich die Werte stark unterscheiden,
  //befindet sich der Zeiger vor dem Sensor
  return (mit_Licht - ohne_Licht);
}

//Schalte den Motor an oder aus
void schalte_Motor(boolean anschalten) {
  digitalWrite(Pin_Apwm, anschalten);
  digitalWrite(Pin_Bpwm, anschalten);
  Motor_ist_an = anschalten;
}

//Schreite einen Schritt nach vorne oder hinten
void schritt(boolean forwaerts) {
  //Berechne, welcher Schritt als nächstes an der Reihe ist
  if (forwaerts) {
    Zustand = ++Zustand % 8;
  } else {
    Zustand = --Zustand % 8;
  }

  //Schalte die Pins je nach Schrittposition
  digitalWrite(Pin_Adir, Schrittfolge[Zustand][0]);
  digitalWrite(Pin_Abrk, !Schrittfolge[Zustand][1]);
  digitalWrite(Pin_Bdir, Schrittfolge[Zustand][2]);
  digitalWrite(Pin_Bbrk, !Schrittfolge[Zustand][3]);
}
